CWD := $(dir $(abspath $(lastword $(MAKEFILE_LIST))))

# Can be overridden to /volume/publish/dev/secureboot/
PREFIX ?= $(CWD)/../bin/publish

lc = $(subst A,a,$(subst B,b,$(subst C,c,$(subst D,d,$(subst E,e,$(subst F,f,$(subst G,g,$(subst H,h,$(subst I,i,$(subst J,j,$(subst K,k,$(subst L,l,$(subst M,m,$(subst N,n,$(subst O,o,$(subst P,p,$(subst Q,q,$(subst R,r,$(subst S,s,$(subst T,t,$(subst U,u,$(subst V,v,$(subst W,w,$(subst X,x,$(subst Y,y,$(subst Z,z,$1))))))))))))))))))))))))))

uc = $(subst a,A,$(subst b,B,$(subst c,C,$(subst d,D,$(subst e,E,$(subst f,F,$(subst g,G,$(subst h,H,$(subst i,I,$(subst j,J,$(subst k,K,$(subst l,L,$(subst m,M,$(subst n,N,$(subst o,O,$(subst p,P,$(subst q,Q,$(subst r,R,$(subst s,S,$(subst t,T,$(subst u,U,$(subst v,V,$(subst w,W,$(subst x,X,$(subst y,Y,$(subst z,Z,$1))))))))))))))))))))))))))

# this follows convention of key labeling
DB_NAME ?= db1
KEY_VER ?= V1
SIGN_CERT ?= $(call uc,$(VERSION_PLATFORM))$(call uc,$(KEY_VER))-$(call uc,$(VERSION_POSTFIX))-$(DB_NAME).pem
SIGN_LABEL ?= $(call uc,$(VERSION_PLATFORM))$(call uc,$(KEY_VER))-$(call uc,$(VERSION_POSTFIX))-$(DB_NAME)-priv

$(info signing key label is [${SIGN_LABEL}])
$(info signing cert filename is [${SIGN_CERT}])

# we default to secure grub memdisk
MEMDISK ?= memdisk-SECURE

UPSTREAM_VERSION = 2.02b
INHOUSE_VERSION ?= bringup
VERSION ?= $(UPSTREAM_VERSION)-$(VERSION_PLATFORM)
ifdef VERSION_POSTFIX
VERSION := $(VERSION)-$(VERSION_POSTFIX)
endif

USER ?= $(shell whoami)
DATE ?= $(shell date +%Y%m%d)
GITREV ?= $(shell git rev-parse --short=10 HEAD)
PKCS11_LIB ?= /usr/safenet/lunaclient/lib/libCryptoki2_64.so
SIGN_SLOT ?= 1
SBSIGN_PKCS11 ?= sbsign_pkcs11

PUBLICATION_ID ?= $(DATE).$(USER).$(GITREV)


GRUB_BUILTIN_MODULES=\
	memdisk\
	part_gpt\
	part_msdos\
	fat\
	iso9660\
	echo\
	efinet\
	test\
	configfile\
	loadenv\
	net\
	normal\
	search\
	verify\
	tpm\
	crypto\
	gcry_rsa\
	gcry_md5\
	gcry_sha256\
	gcry_sha512

grubx64-unsigned.efi: $(CWD)/../bin/$(MEMDISK).iso $(CWD)/../grub-mkimage
	$(CWD)/../grub-mkimage -d $(CWD)/../grub-core -O x86_64-efi\
		$(EMBEDDED_KEYS) \
		-o $@\
		--memdisk $(CWD)/../bin/$(MEMDISK).iso \
		$(GRUB_BUILTIN_MODULES)

.PHONY: sign

sign: grubx64-signed.efi

# If SIGN_KEY is defined, sign w/o PKCS11 backed
# otherwise, we use PKCS11 and expect the correct SIGN_LABEL
# to be defined
grubx64-signed.efi: grubx64-unsigned.efi
ifdef SIGN_KEY
	test -s $(SIGN_KEY) || { echo SIGN_KEY must exist; exit 1; }
	test -s $(SIGN_CERT) || { echo SIGN_CERT must exist; exit 1; }
	sbsign --key $(SIGN_KEY) --cert $(SIGN_CERT) \
		--output $@ $<
else
	@test -n "$(PIN)" || { echo PIN must be defined; exit 1; }
	@test -n "$(SIGN_LABEL)" || { echo SIGN_LABEL must be defined; exit 1; }
	@test -n "$(SIGN_CERT)" || { echo SIGN_CERT must be defined; exit 1; }
	@test -n "$(SIGN_SLOT)" || { echo SIGN_SLOT must be defined; exit 1; }
	@test -s "$(PKCS11_LIB)" || { echo PKCS11_LIB must exist; exit 1; }
	@test -s "$(SBSIGN_PKCS11)" || { echo SBSIGN_PKCS11 must be executable; exit 1; }
	env OPENSSL_ENGINES=/usr/lib/x86_64-linux-gnu/openssl-1.0.0/engines \
		$(SBSIGN_PKCS11) \
		--keylabel "$(SIGN_LABEL)" \
		--lib "$(PKCS11_LIB)" \
		--slot "$(SIGN_SLOT)" \
		--pin "$(PIN)" \
		--cert "$(SIGN_CERT)" \
		--output $@ $<
endif
	sbverify --cert "$(SIGN_CERT)" $@

publish:
	@test -d $(shell dirname $(PREFIX)) || { echo parent dir of prefix \'$(PREFIX)\' must exist; exit 1; }
	@test ! -d $(PREFIX)/grub/$(VERSION)/$(PUBLICATION_ID) || { echo $(PUBLICATION_ID) is already published; exit 1; }
	@mkdir -p $(PREFIX)/grub/$(VERSION)/$(PUBLICATION_ID)
	sh $(CWD)/../publish.sh \
		OBJDIR="$(CWD)/../bin" \
		SRCDIR="$(CWD)/.." \
		KEYDIR="$(CURDIR)" \
		TARGET_DIR="$(PREFIX)/grub/$(VERSION)/$(PUBLICATION_ID)"\
		VERSION="$(VERSION)"

link-pub:
	@test -d $(PREFIX)/grub/$(VERSION)/$(PUBLICATION_ID) || { echo $(PUBLICATION_ID) is not published; exit 1; }
	(cd $(PREFIX)/grub/$(VERSION)/ && rm current 2>/dev/null; ln -s $(PUBLICATION_ID) current )

clean:
	rm -f grubx64-signed.efi grubx64-unsigned.efi
