#!/bin/sh
#
# Copyright (c) 2015, Juniper Networks, Inc.
#

# Empty vars are mandatory, and must be passed in by the caller.
TARGET_DIR=
VERSION=
OBJDIR=bin
SRCDIR=.

Error() {
	echo "ERROR: $@" >&2
	exit 1
}

while :
do
        case "$1" in
        *=*) eval "$1"; shift;;
        *) break;;
        esac
done

test -z "$TARGET_DIR" && Error TARGET_DIR must be passed
test -z "$VERSION" && Error VERSION must be passed
test -z "$KEYDIR" && Error KEYDIR must be passed

case "$VERSION" in
*-prod) # production candidate bootloader
	LIST="${KEYDIR}/grubx64-signed.efi\
		${SRCDIR}/grub-verify\
		${KEYDIR}/gpg.pub\
		${KEYDIR}/grub-trusted.gpg \
		${KEYDIR}/grub-trusted.gpg.psig"
	;;
*-bringup) # bringup bootloader
	LIST="
		${SRCDIR}/grub-verify\
		${KEYDIR}/gpg-dev-sign.pub\
		${KEYDIR}/grubx64-signed.efi\
		${KEYDIR}/grub-trusted.gpg \
		${KEYDIR}/grub-trusted.gpg.psig"
	;;
*-dev) # Development bootloader
	LIST="${KEYDIR}/grubx64-signed.efi\
		${SRCDIR}/grub-verify\
		${KEYDIR}/gpg.pub\
		${KEYDIR}/grub-trusted.gpg \
		${KEYDIR}/grub-trusted.gpg.psig"
	;;
*)
	Error Do not know how to publish $VERSION
	;;
esac

echo Trying to publish ${VERSION}

# assert items in the list exist exist
for item in $LIST; do
	test -s ${item} ||  Error item:${item} doesnt exist
done

# do the publication
for item in $LIST; do
	case $item in
	*grubx64*efi)
		cp ${item} ${TARGET_DIR}/BOOTX64.EFI
		;;
	*grub-trusted*)
		cp -L ${item} ${TARGET_DIR}/
		;;
	*pub)
		cp -L ${item} ${TARGET_DIR}/grub-root.pub
		;;
	*)
		cp ${item} ${TARGET_DIR}/
		;;
	esac
done
